@@ -7,15 +7,40 @@
 ###########################################################################
 import time
 from random import SystemRandom
-import MySQLdb
-import MySQLdb.cursors
+from .config import dbtype, dbfilename
+
+class DatabaseError(Exception):
+    pass
+
+if dbtype == "mysql":
+    try:
+        import MySQLdb
+        import MySQLdb.cursors
+    except:
+        raise DatabaseError('cannot load the MySQL python bindings')
+elif dbtype == "sqlite":
+    try:
+        import sqlite3
+    except:
+        raise DatabaseError('cannot load the SQLite3 python bindings')
+elif dbtype == "mongodb":
+    try:
+        from pymongo import MongoClient, ASCENDING
+    except:
+        raise DatabaseError('cannot load the MongDB python bindings')
+else:
+    print("no database binding specified, switching to the sqlite bindings")
+    try:
+        import sqlite3
+    except:
+        raise DatabaseError('cannot load the SQLite3 python bindings')
 
 from pyeole.log import init_logging
 from pyeole.encode import normalize, remove_accents
-from eoleaaf.config import loglevel, ent_letter, ent_number, multisep
+from .config import loglevel, ent_letter, ent_number, multisep
 # FIXME: utilisé dans des imports ?
-from eoleaaf.config import dbname, dbuser, dbpasswd
-from eoleaaf.config import aaf_exceptions_report_filename
+from .config import dbname, dbuser, dbpasswd
+from .config import aaf_exceptions_report_filename
 
 random = SystemRandom()
 
@@ -29,8 +54,6 @@ def log_error(tablename, modif_type, datestr, db, id_, msg):
     update_journal(tablename, modif_type, datestr, db, id_, 'error')
 
 
-class DatabaseError(Exception):
-    pass
 
 
 # ____________________________________________________________
@@ -43,16 +66,31 @@ class SimpleConnectionWrapper:
     def connect(self, default_cursor=True):
         # wraps the original connection object
         if default_cursor:
-            self.conn = MySQLdb.connect(host="localhost", port=3306, user=self.user,
-                                  passwd=self.passwd, db=self.db)
+            raise Exception("Default database cursor not implemented")
 
-        else:
+        if dbtype == 'mysql':
             self.conn = MySQLdb.connect(host="localhost", port=3306,
                                   user=self.user,
                                   passwd=self.passwd, db=self.db,
+                                  charset='utf8',
                                   cursorclass=MySQLdb.cursors.SSDictCursor)
-            #MySQLdb.cursors.DictCursor)
-        self.conn.autocommit(False)
+            self.conn.autocommit(False)
+        else:
+            if dbtype == 'mysql':
+                # eole-mysql's responsibility
+                pass
+            elif dbtype == 'sqlite':
+                import sqlite3
+                conn = sqlite3.connect(dbfilename)
+                tables = conn.execute("SELECT name FROM sqlite_master WHERE type='table';")
+                if len(list(tables)) == 0:
+                    from os import system
+                    system("/usr/bin/sqlite3 {} < /usr/share/eole/mysql/eole-aaf/gen/eoleaaf.sql".format(dbfilename))
+                self.conn = sqlite3.connect(dbfilename)
+                self.conn.row_factory = sqlite3.Row
+            else:
+                raise DatabaseError("unknown database type : {}".format(dbtype))
+
         self.cursor = self.conn.cursor()
 
     def execute(self, sql, values=None):
@@ -61,8 +99,18 @@ class SimpleConnectionWrapper:
                 self.cursor.execute(sql)
             else:
                 self.cursor.execute(sql, values)
-        except Exception, e:
-            raise DatabaseError("MySQL error, " + str(e))
+        except Exception as err:
+            raise DatabaseError("SQL error, " + str(err))
+
+    def _row_to_dict(self, ret):
+        newlist = []
+        for row in ret:
+            newdict = {}
+            keys = list(row.keys())
+            for idx, value in enumerate(row):
+                newdict[keys[idx]] = value
+            newlist.append(newdict)
+        return newlist
 
     def fetchone(self, sql):
         "retrieves the first entry of a query"
@@ -70,7 +118,11 @@ class SimpleConnectionWrapper:
         if sql.endswith(';'):
             sql = sql[:-1]
         self.cursor.execute(sql + ' LIMIT 1;')
-        ret = list(self.cursor.fetchall())
+        ret = self.cursor.fetchall()
+        if not isinstance(ret, tuple):
+            ret = self._row_to_dict(ret)
+        else:
+            ret = list(ret)
         if ret == []:
             return ret
         return ret[0]
@@ -79,13 +131,20 @@ class SimpleConnectionWrapper:
         "retrieves all entries from the table: tablename"
         sql = "SELECT * FROM " + tablename + " " + condition
         self.cursor.execute(sql)
-        return self.cursor.fetchall()
+        ret = self.cursor.fetchall()
+        if not isinstance(ret, tuple):
+            ret = self._row_to_dict(ret)
+        return ret
 
     def count(self, tablename, condition=""):
         "counts all entries from the table: tablename"
         sql = "SELECT COUNT(*) FROM " + tablename + " " + condition
         self.cursor.execute(sql)
-        return self.cursor.fetchall()[0]["COUNT(*)"]
+        ret = self.cursor.fetchall()
+        if dbtype == 'mysql':
+            return ret[0]["COUNT(*)"]
+        else:
+            return ret[0][0]
 
     def fetchtable(self, tablename):
         """
@@ -105,25 +164,91 @@ class SimpleConnectionWrapper:
         self.cursor.close()
         self.conn.close()
 
-    def commit(self):
+    def commit(self, tablename):
         try:
             self.conn.commit()
-        except Exception, e:
+        except Exception as err:
             log.error("Erreur au commit database")
             self.conn.rollback()
-            raise DatabaseError("MySQL error, " + str(e))
+            raise DatabaseError("MySQL error, " + str(err))
 
     def reset(self):
-        self.execute("SHOW TABLES")
-        for tablename in self.cursor.fetchall():
-            tablename = tablename['Tables_in_eoleaaf']
-            self.execute('TRUNCATE {0}'.format(tablename))
-        self.commit()
+        if dbtype == 'mysql':
+            self.execute("SHOW TABLES")
+            for tablename in self.cursor.fetchall():
+                self.execute('TRUNCATE {0}'.format(tablename['Tables_in_eoleaaf']))
+        else:
+            self.execute("SELECT name FROM sqlite_master WHERE type='table';")
+            for tablename in self.cursor.fetchall():
+                self.execute('DELETE FROM {0}'.format(tablename[0]))
+        self.commit(None)
+
+class SimpleMongoConnectionWrapper(object):
+    def __init__(self):
+        self._datas = []
+
+    def connect(self):
+        # FIXME: gérer l'authentification !!!!
+        self.client = MongoClient('mongodb://localhost:27017')
+        self.db = self.client[dbname]
+        self.collection_user = self.db.user
+        if "ENTEleveGroupes" not in self.collection_user.index_information():
+            self.collection_user.create_index('ENTEleveGroupes')
+
+        if "ENTPersonLogin" not in self.collection_user.index_information():
+            self.collection_user.create_index('ENTPersonLogin', unique=True)
+
+        if "ENTPersonJointure" not in self.collection_user.index_information():
+            self.collection_user.create_index('ENTPersonJointure', unique=True)
+
+        if "UserType" not in self.collection_user.index_information():
+            self.collection_user.create_index('UserType')
+
+        if "uid" not in self.collection_user.index_information():
+            self.collection_user.create_index('uid')
+
+        if "ENTStructureJointure" not in self.db.etablissement.index_information():
+            self.db.etablissement.create_index('ENTStructureJointure')
+
+    def reset(self):
+        #self.client.drop_database(dbname)
+        self.collection_user.remove()
+        self.db.etablissement.remove()
+        self.db.subject.remove()
+        self._datas = []
+
+    def commit(self, tablename):
+        if tablename is not None:
+            #for data in self._datas:
+            #    print data['ENTStructureJointure']
+            #    self.db[tablename].insert(data)
+            self.db[tablename].insert_many(self._datas)
+        self._datas = []
+
+    def close(self):
+        self._datas = []
+        self.client.close()
+
+    def execute(self, data, values=None):
+        self._datas.append(data)
+
+    def fetchone(self, tablename, attr, value):
+        return self.db[tablename].find_one({attr: value})
+
+    def fetchall(self, tablename, attr=None, value=None):
+        if attr is None:
+            return self.db[tablename].find()
+        return self.db[tablename].find({attr: value})
+
 
 # ____________________________________________________________
 # global (and unique) database connection
-db = SimpleConnectionWrapper()
-db.connect(default_cursor=False)
+if not dbtype == "mongodb":
+    db = SimpleConnectionWrapper()
+    db.connect(default_cursor=False)
+else:
+    db = SimpleMongoConnectionWrapper()
+    db.connect()
 # ____________________________________________________________
 
 def replace_cars(chaine, chars):
@@ -132,44 +257,6 @@ def replace_cars(chaine, chars):
     """
     if chaine is None:
         return ''
-    if type(chaine) == unicode:
-        chaine = chaine.encode('UTF-8')
-    for old, new in chars.items():
-        chaine = chaine.replace(old, new)
-    return chaine
-
-def replace_cars2(chaine):
-    """
-    fonction de suppression des caractères spéciaux les plus courants
-    """
-    if chaine is None:
-        return ''
-    if type(chaine) == unicode:
-        chaine = chaine.encode('UTF-8')
-    chars = {'¥' : 'Y', 'µ' : 'u', 'À' : 'A', 'Á' : 'A',
-             'Â' : 'A', 'Ã' : 'A', 'Ä' : 'A', 'Å' : 'A',
-             'Æ' : 'A', 'Ç' : 'C', 'È' : 'E', 'É' : 'E',
-             'Ê' : 'E', 'Ë' : 'E', 'Ì' : 'I', 'Í' : 'I',
-             'Î' : 'I', 'Ï' : 'I', 'Ð' : 'D', 'Ñ' : 'N',
-             'Ò' : 'O', 'Ó' : 'O', 'Ô' : 'O', 'Õ' : 'O',
-             'Ö' : 'O', 'Ø' : 'O', 'Ù' : 'U', 'Ú' : 'U',
-             'Û' : 'U', 'Ü' : 'U', 'Ý' : 'Y', 'ß' : 's',
-             'à' : 'a', 'á' : 'a', 'â' : 'a', 'ã' : 'a',
-             'ä' : 'a', 'å' : 'a', 'æ' : 'a', 'ç' : 'c',
-             'è' : 'e', 'é' : 'e', 'ê' : 'e', 'ë' : 'e',
-             'ì' : 'i', 'í' : 'i', 'î' : 'i', 'ï' : 'i',
-             'ð' : 'o', 'ñ' : 'n', 'ò' : 'o', 'ó' : 'o',
-             'ô' : 'o', 'õ' : 'o', 'ö' : 'o', 'ø' : 'o',
-             'ù' : 'u', 'ú' : 'u', 'û' : 'u', 'ü' : 'u',
-             'ý' : 'y', 'ÿ' : 'y', '~B': 'e', '°' : '.', 'A©' : 'e',
-             'Š' : '', 'þ': 'e', '²': '',
-#             '&#176;': '', '&#194;':'', '&#352;':'', '&#233;':'e',
-#             '&#226;': 'a', '&#233;': 'e', '&#232;': 'e',
-#             '&0xc3;': '', '&#231;': 'c', '&apos;': " ", '&#254;': 'e',
-#              '&amp;': '', '&#224;':'', '&#201;': 'E', '&quot;': '',
-#              '&#226;': 'a', '&#235;': 'e', "'": "", 'A©':'e',
-#              '&#195;': 'a', '&#139;': '',
-            }
     for old, new in chars.items():
         chaine = chaine.replace(old, new)
     return chaine
@@ -195,60 +282,67 @@ def replace_dangerous_cars(chaine):
             }
     return replace_cars(chaine, chars)
 
-def replace_bad_cars(chaine):
-    """
-    remplace les caractères invalides
-    """
-    longchars = {
-            '\xc3\x82\xc2\xb0':'O',
-            '\xc3\x83\xc2\x82':'A',
-            '\xc3\x83\xc2\x87':'C',
-            '\xc3\x83\xc2\x88':'E',
-            '\xc3\x83\xc2\x89':'E',
-            '\xc3\x83\xc2\x8a':'E',
-            '\xc3\x83\xc2\x8b':'A',
-            '\xc3\x83\xc2\x91':'N',
-            '\xc3\x83\xc2\x94':'O',
-            }
-    shortchars = {
-            '\xc3\xa8' : 'E',
-            '\xc3\xaf' : 'I',
-            '\xc3\x82' : 'A',
-            '\xc3\x83' : 'E',
-            '\xc3\x87' : '?',
-            '\xc3\x88' : 'E',
-            '\xc3\x89' : 'E',
-            '\xc3\x8b' : 'E', #was ê maj
-            '\xc3\x8f' : 'A',
-            # ---------------- #
-            '\xc3\x8a' : 'e', #was è
-            '\xc3\xa7' : 'c', #was ç
-            '\xc3\xa9' : 'e', #was é
-            '\xc3\xaa' : 'e', #was ê
-            '\xc3\xab' : 'e', #was ë
-            '\xc3\xbb' : 'u',
-            }
-
-#             '&#176;': '', '&#194;':'', '&#352;':'', '&#233;':'e',
-#             '&#226;': 'a', '&#233;': 'e', '&#232;': 'e',
-#             '&0xc3;': '', '&#231;': 'c', '&#254;': 'e',
-#             '&#224;':'', '&#201;': 'E',
-#             '&#226;': 'a', '&#235;': 'e',
-#             '&#195;': 'a', '&#139;': '',
-#             '\xc2\x88' : '',
-#             '\xc2\x89' : '',
-#             '\xc2\x8a' : 'I',
-#             '\xc2\x8b' : '',
-#             '\xc2\xb0': '',
-#             '\xc3\x82' : 'O',
-    return replace_cars(replace_cars(chaine, longchars), shortchars)
-
 def clean_empty_keys(empty_keys_dict):
     "delete empty keys to prepare a clen sql insert"
-    for key, value in empty_keys_dict.items():
+    for key, value in list(empty_keys_dict.items()):
         if value in ['', None]:
-            del(empty_keys_dict[key])
+            del empty_keys_dict[key]
     return empty_keys_dict
+## ______________________________________________________________________
+## Mongo Engine data base ORM
+#
+#class Group(Document):
+#    description = StringField(required=True)
+#
+#class Student(Document):
+#    id = StringField(primary_key=True, required=True)
+#    uid = StringField()
+#    userPassword = BinaryField()
+#    ENTPersonLogin = StringField()
+#    ENTEleveFiliere = StringField()
+#    ENTEleveTransport = StringField()
+#    ENTEleveEnseignements = StringField()
+#    ENTEleveStructRattachId = StringField()
+#    ENTEleveClasses = StringField()
+#    ENTEleveMEF = StringField()
+#    ENTPersonJointure = StringField()
+#    ENTEleveNivFormation = StringField()
+#    ENTEleveStatutEleve = StringField()
+#    ENTEleveAutoriteParentale = StringField()
+#    ENTElevePere = StringField()
+#    ENTEleveQualitePersRelEleve2 = StringField()
+#    ENTEleveQualitePersRelEleve1 = StringField()
+#    ENTEleveMere = StringField()
+#    ENTEleveGroupes = ListField(ReferenceField(Group))
+#    ENTPersonDateNaissance = StringField()
+#    ENTElevePersRelEleve2 = StringField()
+#    ENTElevePersRelEleve1 = StringField()
+#    ENTEleveParents = StringField()
+#    ENTEleveBoursier = StringField()
+#    ENTEleveLibelleMEF = StringField()
+#    personalTitle = StringField()
+#    ENTPersonNomPatro = StringField()
+#    sn = StringField()
+#    ENTPersonStructRattach = StringField()
+#    ENTPersonAutresPrenoms = StringField()
+#    givenName = StringField()
+#    ENTEleveRegime = StringField()
+#    ENTEleveCodeEnseignements = StringField()
+#    ENTElevePersRelEleve = StringField()
+#    FieldModifiedStatus = BooleanField()
+#    FieldActionType = StringField()
+#    FieldFromXMLImportActionDate = DateTimeField()
+#    FieldToLDIFFExportActionDate = DateTimeField()
+
+
+class MongoDBGenerator(object):
+
+    def insert(self, insert, table, paramstyle=False):
+        # typing the collection
+        if table not in ['etablissement', 'subject']:
+            insert['UserType'] = table
+        return insert, None
+
 
 class MinimalSQLGenerator:
     "minimal sql generation"
@@ -270,17 +364,20 @@ class MinimalSQLGenerator:
         begin_insert = "INSERT into %s (" % table
         insert_decls = begin_insert + ", ".join(insert.keys()) + ') '
         if paramstyle:
-            values = insert.values()
-            insert_vals = ['%s' for val in values]
+            values = list(insert.values())
+            if dbtype == 'mysql':
+                tmpl = '%s'
+            else:
+                tmpl = '?'
+            insert_vals = [tmpl for val in values]
         else:
-            insert_vals = ['"%s"' % val for val in insert.values()]
+            insert_vals = [f'"{val}"' for val in insert.values()]
         commas = ', '.join(insert_vals)
         begin_vals = "VALUES ("
         sql_insert = insert_decls + begin_vals + commas + " ) ;"
         if paramstyle:
             return sql_insert, values
-        else:
-            return sql_insert
+        return sql_insert
 
     def delete(self, id_name, id_value, table):
         """
@@ -317,10 +414,14 @@ class MinimalSQLGenerator:
            (si paramstyle == True)
         """
         update = clean_empty_keys(update)
-        sql_update = "UPDATE %s SET " % table
+        sql_update = f"UPDATE {table} SET "
         if paramstyle:
-            values = update.values()
-            where_clause = [self._where(key, '%s', sep='') for key, value in update.items()]
+            if dbtype == 'mysql':
+                tmpl = '%s'
+            else:
+                tmpl = '?'
+            values = list(update.values())
+            where_clause = [self._where(key, tmpl, sep='') for key, value in update.items()]
         else:
             where_clause = [self._where(key, value, sep='"') for key, value in update.items()]
         setstatus = ", ".join(where_clause)
@@ -329,10 +430,9 @@ class MinimalSQLGenerator:
         sql_update += where + " ;"
         if paramstyle:
             return sql_update, values
-        else:
-            return sql_update
+        return sql_update
 
-    def where(self, field, key, value) :
+    def where(self, field, key, value):
         """
         >>> sqlgen.where('uid', 'id', 'idresp')
         select uid where id=idresp
@@ -357,17 +457,13 @@ class MinimalSQLGenerator:
         # XXX concatener une serie de conditions sur les clefs
         return "{0}={2}{1}{2}".format(key, value, sep)
 
-# ____________________________________________________________
-# global sql generator object for uses from inside the lib
-sqlgen = MinimalSQLGenerator()
-
 # ____________________________________________________________
 def make_datetime():
     now = time.localtime(time.time())
     return time.strftime("%Y-%m-%d %H:%M:%S ", now)
 
 
-def modif_fields(notify, modif_type="UPDATE",
+def modif_fields(notify, sqlgen, modif_type="UPDATE",
                 tablename="eleve", fromxml=True,
                 db=None, id_column='id'):
     """
@@ -416,7 +512,7 @@ def modif_fields(notify, modif_type="UPDATE",
                     db.execute(sql, values)
                 else:
                     db.execute(sql)
-            except Exception, msg:
+            except Exception as msg:
                 log.info("Erreur au moment du storage du delete dans le journal : " + sql)
             sql = sqlgen.delete(id_column, action_id, tablename)
     else:
@@ -431,7 +527,7 @@ def modif_fields(notify, modif_type="UPDATE",
             db.execute(sql, values)
         else:
             db.execute(sql)
-    except Exception, msg:
+    except Exception as msg:
         if "Duplicate entry" in str(msg):
             log.info("Cette entrée est déjà présente dans la base de données au moment de {0} pour {1} avec ID {2}.".format(
                  modif_type, tablename, action_id.encode("utf-8")))
@@ -449,6 +545,7 @@ def update_journal(tablename, modif_type, datestr, db, id_, level):
     :tablename: the table where the modification has been made
     :actiontype: in ['CREATE', 'UPDATE', 'DELETE']
     """
+    return
     notify = dict()
     notify['ActionDate'] = datestr
     notify['ActionId'] = id_
@@ -517,7 +614,7 @@ def random_identifier(nblength=4, namelength=2):
 #    write_id_file(filename, rnd)
 #    return rnd.split("-")
 
-def make_unique_random_id():
+def make_unique_random_id(rnd_buffer, sqlgen):
     """generates a unique id for the 'ENTPersons'
     it uses the primary key in the 'randomids' sql table
     while an id exists in the table, it is calculated again
@@ -528,10 +625,19 @@ def make_unique_random_id():
     while rndfind:
         idx += 1
         try:
-            sql = sqlgen.insert({'id':rnd}, 'randomids')
-            db.execute(sql)
-            rndfind = False
-        except Exception, msg:
+            if dbtype == "mongodb":
+                if rnd in rnd_buffer or db.fetchone("user", "uid", rnd):
+                    rnd = random_identifier()
+                else:
+                    rndfind = False
+                    rnd_buffer.append(rnd)
+            else:
+                sql = sqlgen.insert({'id': rnd}, 'randomids')
+                db.execute(sql)
+                rndfind = False
+        except DatabaseError:
+            # Force commit if insert failed
+            db.commit(None)
             rnd = random_identifier()
         if idx == 10:
             raise Exception('cannot generate ID')
@@ -540,29 +646,33 @@ def make_unique_random_id():
 
 #______________________________________________________________________________
 
-def get_new_id():
+def get_new_id(rnd_buffer, sqlgen):
     """
     Renvoie un identifiant unique
     de la forme "LxxCiiii" avec :
     o L et C : Lettre et chiffre du code projet ENT
     o xx et iiii : 2 lettres et 4 chiffres à générer pour chaque entrée
     """
-    letters, numbers = make_unique_random_id()
+    letters, numbers = make_unique_random_id(rnd_buffer, sqlgen)
     return "{0}{1}{2}{3}".format(ent_letter, letters, ent_number, numbers)
 
 #______________________________________________________________________________
 
-def gen_user_login(surname, givenname, uid, usertype, user_id):
+def gen_user_login(surname, givenname, uid, usertype, user_id, sqlgen, login_buffer):
     """
     génération du login
     le login est de type 'prenom.nomXX'
     """
+    def gen_login_num(num):
+        if result < 9:
+            loginnum = "0" + str(num)
+        else:
+            loginnum = str(num)
+        return loginnum
+
     givenname = normalize(givenname)
     surname = normalize(surname)
     # C'EST CONTRAIRE AU SDET - on cherche quand même a attribuer un login a l'utilisateur
-    #if surname is None:
-    #    raise Exception('Impossible de générer un login avec le '
-    #                    'nom {0}'.format(surname))
     loginalpha = ""
     if surname is not None or surname != "":
         if (givenname is not None and givenname.strip() not in ['-', '_', '']
@@ -573,38 +683,44 @@ def gen_user_login(surname, givenname, uid, usertype, user_id):
             loginalpha = surname.lower()
         loginalpha = remove_accents(loginalpha).replace("'", "").replace(" ", "").replace("-", "").replace("_", "")
 
-    result = None
-    try:
-        where = "where ENTPersonLoginAlpha like '{0}';".format(loginalpha)
-        result = db.count('personlogin', where)
-    except Exception, e:
-        raise Exception('Impossible de générer un login avec le '
-                        'nom {0} et le prénom {1}'.format(surname, givenname))
-    if result < 9:
-        loginnum = "0" + str(result+1)
-    else:
-        loginnum = str(result+1)
-
     if loginalpha == "":
         # C'EST CONTRAIRE AU SDET - on cherche quand même a attribuer un login a l'utilisateur
         # on lui attribut un login correspondant au user id au sens AAF (pas l'uid généré)
         loginalpha = user_id
-        login = loginalpha + loginnum
+        write_report = True
+    else:
+        write_report = False
+    result = 0
+    rndfind = True
+    idx = 0
+    while rndfind:
+        try:
+            idx += 1
+            if idx == 101:
+                raise Exception('cannot generate login')
+            loginnum = gen_login_num(idx)
+            login = loginalpha + loginnum
+            if dbtype != "mongodb":
+                sql = sqlgen.insert({'ENTPersonLoginAlpha': loginalpha,
+                        'ENTPersonLoginNum': loginnum,
+                        'DateCreation': make_datetime(),
+                        'uid': uid,
+                        'usertype': usertype}, 'personlogin')
+                db.execute(sql)
+            else:
+                if login in login_buffer or db.fetchone("user", "ENTPersonLogin", login):
+                    continue
+                login_buffer.append(login)
+            rndfind = False
+        except DatabaseError:
+            pass
+        except Exception as msg:
+            raise Exception('Impossible de créer le login pour {0} : {1}'.format(loginalpha, msg))
+    if write_report:
         # on ecrit un rapport sur les cas exceptionnels dans un fichier texte
-        fh = file(aaf_exceptions_report_filename, 'a+')
+        fh = open(aaf_exceptions_report_filename, 'a+')
         fh.write("l'utilisateur avec l'id: {0} se voit affecter le login : {1}\n".format(user_id, login))
         fh.close()
-
-    login = loginalpha + loginnum
-    try:
-        sql = sqlgen.insert({'ENTPersonLoginAlpha': loginalpha,
-                'ENTPersonLoginNum': loginnum,
-                'DateCreation': make_datetime(),
-                'uid': uid,
-                'usertype': usertype}, 'personlogin')
-        db.execute(sql)
-    except Exception, msg:
-        raise Exception('Impossible de créer le login {0}'.format(login))
     return login
 
 def parse_multi(attr, update=False):
@@ -617,36 +733,20 @@ def parse_multi(attr, update=False):
     balise = attr.attrib['name']
     vals = []
     for val in attr.findall('value'):
-        vals.append(replace_dangerous_cars(val.text))
-    try:
-        value = unicode(multisep.join(vals))
-    except:
-        # remplacement des caractères uniquement en cas de problème
-        #new_vals = [replace_bad_cars(val) for val in vals]
-        #try:
-        #value = unicode(multisep.join(vals), 'utf-8').encode('ASCII', 'ignore')
-        myval = multisep.join(vals)
-        myvals = []
-        for val in vals:
-            if type (val) == unicode:
-                newval = str(unicode.encode(val, "utf-8")).decode('utf-8')
-                myvals.append(newval)
-            else:
-                myvals.append(val)
-        value = multisep.join(myvals)
-        #value = str(unicode.encode(multisep.join(vals), "utf-8")).decode('utf-8')
-        #except Exception, e:
-        #    log.error(e)
-        #    log.error("Erreur au traitement de la balise {0}".format(balise))
-        #    log.error(vals)
-        #    log.error("Avec tentative de réécriture en :")
-        #    log.error(new_vals)
-        #    value = 'inconnu'
-        #raise TypeError("Mauvais encoding dans la balise {0}".format(balise))
-    if update:
-        return balise, value, attr.attrib['operation']
+        val = replace_dangerous_cars(val.text)
+        if not isinstance(val, str):
+            val = val.decode('utf-8')
+        # there should be no duplicates in values (#28562)
+        if val not in vals:
+            vals.append(val)
+    if dbtype == 'mongodb':
+        if len(vals) == 1:
+            vals = vals[0]
     else:
-        return balise, value
+        vals = str(multisep.join(vals))
+    if update:
+        return balise, vals, attr.attrib['operation']
+    return balise, vals
 
 def pwgen():
     "convenient password generator"
